//+------------------------------------------------------------------+
//|                                       Trix of log of a price.mq4 |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "copyleft mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  clrDeepSkyBlue
#property indicator_color2  clrSandyBrown
#property indicator_color3  clrSandyBrown
#property indicator_color4  clrRed
#property indicator_style4  STYLE_DOT
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property strict

//
//
//
//
//

enum enColorChange
{
   cch_onZero,  // Change colors on zero cross
   cch_onSign,  // Change colors on signal cross
   cch_onSlope  // Change colors on slope change
};
extern int                TrixPeriod      = 9;            // Trix period
extern ENUM_APPLIED_PRICE TrixPrice       = PRICE_CLOSE;  // Price
extern int                SignalPeriod    =  9;           // Signal line period
extern ENUM_MA_METHOD     SignalMethod    = MODE_LWMA;    // Signal line method
extern enColorChange      ChangeOn        = cch_onSign;   // Change colors (and arrows and alerts) on :
extern bool               AlertsOn        = false;        // Turn alerts on?
extern bool               AlertsOnCurrent = false;        // Alerts on a current (still opened) bar?
extern bool               AlertsMessage   = true;         // Alerts should display pop-up message?
extern bool               AlertsSound     = false;        // Alerts should play alert sound
extern bool               AlertsEmail     = false;        // Alerts should send an email?
extern bool               AlertsNotify    = false;        // Alerts should send push notification?
extern bool               arrowsVisible   = false;        // Arrows visible?
extern string             arrowsID        = "TrixArrows"; // Arrows unique ID
extern double             arrowsUpperGap  = 1.0;          // Arrows down gap
extern double             arrowsLowerGap  = 1.0;          // Arrows up gap
extern color              arrowsUpColor   = clrDeepSkyBlue; // Arrows up color
extern color              arrowsDnColor   = clrOrangeRed; // Arrows down color
extern int                arrowsUpCode    = 241;          // Arrows up code
extern int                arrowsDnCode    = 242;          // Arrows down code

//
//
//
//
//

double TrixBuffer[],TrixBufferda[],TrixBufferdb[];
double SignBuffer[];
double EmaBuffer[],state[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(6);
      SetIndexBuffer(0,TrixBuffer);
      SetIndexBuffer(1,TrixBufferda);
      SetIndexBuffer(2,TrixBufferdb);
      SetIndexBuffer(3,SignBuffer);
      SetIndexBuffer(4,EmaBuffer);
      SetIndexBuffer(5,state);
   IndicatorShortName("Trix ("+(string)TrixPeriod+")");
   return(0);
}

int deinit()
{
   string lookFor       = arrowsID+":";
   int    lookForLength = StringLen(lookFor);
      for (int i=ObjectsTotal()-1; i>=0; i--) 
      {
         string name = ObjectName(i);  if (StringSubstr(name,0,lookForLength) == lookFor) ObjectDelete(name);
      }
   return(0); 
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0)  counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //

   if (state[limit]== 1) CleanPoint(limit,TrixBufferda,TrixBufferdb);
   for(int i=limit; i>=0; i--)
   {
      double price = MathLog(iMA(NULL,0,1,0,MODE_SMA,TrixPrice,i));
         TrixBufferda[i] = EMPTY_VALUE;
         TrixBufferdb[i] = EMPTY_VALUE;
         EmaBuffer[i]    = iEma(iEma(iEma(price,TrixPeriod,i,0),TrixPeriod,i,1),TrixPeriod,i,2);
         if (i<(Bars-1))
               TrixBuffer[i] = (EmaBuffer[i]-EmaBuffer[i+1])*10000;
         else  TrixBuffer[i] = 0;
   }
   for (int i=limit;i>=0;i--)
   {
      SignBuffer[i] = iMAOnArray(TrixBuffer,0,SignalPeriod,0,SignalMethod,i);
      if (i>=(Bars-1)) continue;

      //
      //
      //
      //
      //
            
      state[i] = state[i+1];
      switch(ChangeOn)
      {
         case cch_onSign: 
            if (TrixBuffer[i]>SignBuffer[i]) state[i] =  1;
            if (TrixBuffer[i]<SignBuffer[i]) state[i] = -1;
            break;
         case cch_onZero: 
            if (TrixBuffer[i]>0) state[i] =  1;
            if (TrixBuffer[i]<0) state[i] = -1;
            break;
         default :            
            if (TrixBuffer[i]>TrixBuffer[i+1]) state[i] =  1;
            if (TrixBuffer[i]<TrixBuffer[i+1]) state[i] = -1;
      }            
      if (arrowsVisible)
      {
         deleteArrow(Time[i]);
         if (state[i] != state[i+1])
         {
            if (state[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
            if (state[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
         }
      }            
      if (state[i]==-1) PlotPoint(i,TrixBufferda,TrixBufferdb,TrixBuffer);
   }

   //
   //
   //
   //
   //
         
   if (AlertsOn)
   {
      int forBar = 1; if (AlertsOnCurrent) forBar = 0;
      if (state[forBar]!=state[forBar+1])
      {
         if (state[forBar]== 1) doAlert("UP");
         if (state[forBar]==-1) doAlert("DOWN");
      }         
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workEma[][3];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
      
   workEma[r][instanceNo] = price;
   double alpha = 2.0 / (1.0+period);
   if (r>0)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," trix trend changed to ",doWhat);
          if (AlertsMessage) Alert(message);
          if (AlertsEmail)   SendMail(doWhat,message);
          if (AlertsNotify)  SendNotification(message);
          if (AlertsSound)   PlaySound("alert2.wav");
   }
}

void drawArrow(int i,color theColor,int theCode,bool up, string add="")
{
   string name = arrowsID+":a:"+add+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
         ObjectSet(name,OBJPROP_HIDDEN,true);
         ObjectSet(name,OBJPROP_SELECTABLE,false);
}
void deleteArrow(datetime time, string add = "") { string lookFor = arrowsID+":a:"+add+(string)time; ObjectDelete(lookFor); }

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}